program SvComEx22;

uses
    WinSvc,
    SvCom_WinSvc,
    SvCom_NTService,
    Windows,
    IniFiles,
    Registry,
    SysUtils,
    svcExample22 in 'svcExample22.pas' {svcServiceBase: TNtService};

{$R *.RES}

const RegKey='Software\SvCom\Example22';

procedure CreateNewServices;
var Ini: TIniFile;
    i, Count: Integer;
    Svc: TNTService;
    R: TRegistry;
begin
    Ini:=TIniFile.Create(ChangeFileExt(ParamStr(0), '.INI'));

    R:=TRegistry.Create;
    R.RootKey:=HKEY_LOCAL_MACHINE;
    R.OpenKey(RegKey,True);

    Count:=Ini.ReadInteger('Options','Count',1);
    R.WriteInteger('Count',Count);

    for i:=1 to Count do
    begin
        Svc:=TsvcServiceBase.Create(Application);

        Svc.Name:=Ini.ReadString('Options','Name'+IntToStr(i),'');

        if Svc.Name = '' then
            raise Exception.CreateFmt('Service name missed for the service %d',[i]);

        R.WriteString('Name'+IntToStr(i),Svc.Name);

        Svc.DisplayName:=Ini.ReadString('Options','DisplayName'+IntToStr(i),'');

        if Svc.DisplayName = '' then
            raise Exception.CreateFmt('Service display name missed for the service %d',[i]);

        R.WriteString('DisplayName'+IntToStr(i),Svc.DisplayName);
    end;

    R.Free;

    Ini.Free;
end;

procedure CreateOldServices;
var R: TRegistry;
    i, Count: Integer;
    Svc: TNTService;
begin
    R:=TRegistry.Create;
    R.RootKey:=HKEY_LOCAL_MACHINE;
    R.OpenKey(RegKey,True);

    Count:=R.ReadInteger('Count');

    for i:=1 to Count do
    begin
        Svc:=TsvcServiceBase.Create(Application);

        Svc.Name:=R.ReadString('Name'+IntToStr(i));

        Svc.DisplayName:=R.ReadString('DisplayName'+IntToStr(i));
    end;

    R.Free;
end;

procedure StartService(ServiceName: String);
var
    schService,
    schSCManager: DWORD;
    p: PChar;
begin
    p:=nil;
    schSCManager:= SvOpenSCManager(nil, Nil, SC_MANAGER_ALL_ACCESS);
    if schSCManager = 0 then RaiseLastWin32Error;
    try

            schService:=SvOpenService(schSCManager,PChar(ServiceName),SERVICE_ALL_ACCESS);
            if schService = 0 then RaiseLastWin32Error;

        try
            if not SvStartService(schService,0,p) then RaiseLastWin32Error;
        finally
            SvCloseServiceHandle(schService);
        end;
    finally
        SvCloseServiceHandle(schSCManager);
    end;
end;

procedure StopService(ServiceName: String);
var
    schService,
    schSCManager: DWORD;
    SS: _Service_Status;
begin
    schSCManager:= SvOpenSCManager(nil, Nil, SC_MANAGER_ALL_ACCESS);
    if schSCManager = 0 then RaiseLastWin32Error;

    try
        schService:=SvOpenService(schSCManager,PChar(ServiceName),SERVICE_ALL_ACCESS);
        if schService = 0 then RaiseLastWin32Error;

        try
            if not SvControlService(schService,SERVICE_CONTROL_STOP,SS) then RaiseLastWin32Error;
        finally
            SvCloseServiceHandle(schService);
        end;
    finally
        SvCloseServiceHandle(schSCManager);
    end;
end;


var ServiceName: String;
begin
    Application.Initialize;

    if SvComFindParameter('start', ServiceName) then
    begin
        StartService(ServiceName);
        exit;
    end;

    if SvComFindParameter('stop', ServiceName) then
    begin
        StopService(ServiceName);
        exit;
    end;

    if SvComFindCommand('ReInstall') then
        raise Exception.Create('/REINSTALL switch is not supported');

    if SvComFindCommand('Install') then
        CreateNewServices
    else
        CreateOldServices;

    Application.Run;
end.
